eval 'exec perl -wS $0 ${1+"$@"}'
    if 0;
# /usr/bin/perl -w
# failure tolerant mount wrapper
# $Id: ftmount.pl,v 1.3 2006/07/01 17:30:42 langha_l Exp $
#
# (cdw) Lars Langhans 2006
# Distributed under the terms of the GNU General Public License v2

use strict;
use File::Basename;
use Getopt::Long;
use Cwd;
use English;
use File::Path;
use Cwd 'chdir';
my $cwd = getcwd();

# Globals
our $verbose;
our $list;
our $help;
our $version;
our $version_info = '$Revision: 1.3 $';
our $mount;
our $unmount;
our $sUnmountFile = "/mnt/.unmountdevices";
our $nMountNumber = -1; # all

# Prototypes
sub print_usage(*);
sub identifyDiskAndPartions();
sub identifyAlreadyMounted();
sub identifyInFstab();
sub alreadyMounted($);
sub getMountpoint($);
sub getDiskSize($);
sub getPossibleMountpoint($);
sub showDiskSize($);
sub mountPartion($$);

# Parse command line options
if (!GetOptions(
                "verbose"      => \$verbose,
                "mount"        => \$mount,
                "unmount"      => \$unmount,
                "list"         => \$list,
                "help"         => \$help,
                "version"      => \$version
                ))
{
    print_usage(*STDERR);
    exit(1);
}

# Check for help option
if ($help)
{
    print_usage(*STDOUT);
    exit(0);
}
# Check for version option
if ($version)
{
    print("$version_info\n");
    exit(0);
}

if ($UID != 0)
{
    print "You are not 'root' user.\n";
    exit(1);
}

if ($#ARGV >= 0)
{
    $nMountNumber = $ARGV[0];
}

if ($unmount)
{
    local *UNMOUNT;
    if ( -e "$sUnmountFile")
    {
        `$sUnmountFile`;
        # if (open(UNMOUNT, $sUnmountFile))
        # {
        #     print "Unmount all devices, which mount with this tool.\n";
        #     my $line;
        #     while ($line=<UNMOUNT>)
        #     {
        #         chomp($line);
        #         if (! ($line =~ /^\#/ ))
        #         {
        #             `$line`;
        #         }
        #     }
        #     close(UNMOUNT);
        # 
        # } 
        unlink($sUnmountFile);
    }
    else
    {
        print "Nothing to unmount.\n";
    }
    exit(0);
}

my @Disks;
my @Partions;

identifyDiskAndPartions();

my @AlreadyMount;
identifyAlreadyMounted();

my @InFstab;
identifyInFstab();

# sample, show all mountpoints from /etc/fstab
# my $Fstab;
# foreach $Fstab (@InFstab)
# {
#     print "$Fstab->{Source}\n";
# }

# sample, show all disks
# my $Disk;
# foreach $Disk (@Disks)
# {
#     print "$Disk->{Source}\n";
# }

# sample, show all already mounted devices
# my $aMount;
# foreach $aMount (@AlreadyMount)
# {
#     print "$aMount->{Source}\n";
# }

print "Currently mounted partions:\n";
my $Partion;
my $sDiskSize;
my $sPartionSize;
foreach $Partion (@Partions)
{
    my $sMountpoint = getMountpoint( $Partion->{Source} );
    $sDiskSize = showDiskSize( getDiskSize($Partion->{Source}) );
    $sPartionSize = showDiskSize($Partion->{Size});
    if ($sMountpoint)
    {
        print "$Partion->{Source} $sPartionSize $sDiskSize '$sMountpoint'\n";
    }
}
print "\n";

print "Currently not mounted partions:\n";
for (my $i=0;$i<2;$i++)
{
    my $nCount = 1;
    foreach $Partion (@Partions)
    {
        if (alreadyMounted($Partion->{Source}) == 0)
        {
            my $sPossibleMountpoint = getPossibleMountpoint( $Partion->{Source} ); # out of /etc/fstab
            $sDiskSize = showDiskSize( getDiskSize($Partion->{Source}) );
            if ($i == 0) { print "[$nCount] $Partion->{Source}"; }
            
            $sPartionSize = showDiskSize($Partion->{Size});
            if ($i == 0) { print " $sPartionSize $sDiskSize"; }
            if ($sPossibleMountpoint)
            {
                if ($i == 0) { print " '$sPossibleMountpoint'"; }
            }
            # mount is possible
            if ($nMountNumber == $nCount ||
                $nMountNumber == -1)
            {
                if ( $mount )
                {
                    if ($i == 0)
                    {
                        print " mounted";
                    }
                    else
                    {
                        mountPartion($Partion, $sPossibleMountpoint);
                    }
                }
            }
            $nCount++;
            if ($i == 0) { print "\n"; }
        }
    }
}
if ($mount)
{
    # change access rights
    `chmod u+x $sUnmountFile`;

    print "\n";
    print "To unmount all mounted devices, just call this tool with parameter '-u'\n";
    exit(0);
}

# if ($list)
# {
#     exit(0);
# }

sub mountPartion($$)
{
    my $Partion = shift;
    my $sPossibleMountpoint = shift;

    # mount to possible mountpoint
    # or /mnt/devicename
    # copy the new '/mnt/devicename' into a special file for unmounting
    my $nCreateDirectoryForMountpoint = 0;
    if ($sPossibleMountpoint)
    {
    }
    else
    {
        # no possible mountpont is given, create one in /mnt/devicename
        my $devicename = $Partion->{Source};
        $devicename =~ s/^\/dev\///; # remove '/dev/'
        my $sNewMountpoint = "/mnt/$devicename";
        $sPossibleMountpoint = $sNewMountpoint;
        if ( ! -d $sPossibleMountpoint)
        {
            `mkdir -p $sPossibleMountpoint`;
            $nCreateDirectoryForMountpoint = 1;
        }
        print " => '$sPossibleMountpoint'";
    }
    local *UNMOUNT;
    open(UNMOUNT, ">>$sUnmountFile");
    print UNMOUNT "umount $sPossibleMountpoint\n";
    if ($nCreateDirectoryForMountpoint == 1)
    {
        print UNMOUNT "rmdir $sPossibleMountpoint\n";
    }
    close(UNMOUNT);
    my $sExtraOption = "";
    if ($Partion->{Type} =~ /FAT/)
    {
        $sExtraOption = "quiet,shortname=winnt";
    }
    `mount -a $Partion->{Source} $sPossibleMountpoint -o $sExtraOption,auto,noatime`;
}
# ------------------------------------------------------------------------------
sub showDiskSize($)
{
    my $nSize = shift;
    my $sNewSize;
    my $sType;

    if ($nSize > 1000000000)
    {
        # giga bytes
        $sNewSize = sprintf ("%.1f", $nSize / 1000000000);
        $sType = "GB";
    }
    elsif ($nSize > 1000000)
    {
        # mega bytes
        $sNewSize = sprintf("%.1f", $nSize / 1000000);
        $sType = "MB";
    }
    elsif ($nSize > 1000)
    {
        # kilo bytes
        $sNewSize = sprintf("%.1f", $nSize / 1000);
        $sType = "KB";
    }
    else
    {
        # bytes
        $sNewSize = $nSize;
        $sType = "B";
    }
    my $sRet = "$sNewSize$sType";
    return sprintf("%8s", $sRet);
}
# ------------------------------------------------------------------------------
# return 1 if the given partion string (/dev/something) already in @AlreadyMount array
# bad Performance O(n)
sub alreadyMounted($)
{
    my $sPartion = shift;
    my $aMount;
    foreach $aMount (@AlreadyMount)
    {
        if ($aMount->{Source} eq $sPartion)
        {
            return 1;
        }
    }
    return 0;
}
# ------------------------------------------------------------------------------
# return mountpoint if the given partion string (/dev/something) already in @AlreadyMount array
# bad Performance O(n)
sub getMountpoint($)
{
    my $sPartion = shift;
    my $aMount;
    foreach $aMount (@AlreadyMount)
    {
        if ($aMount->{Source} eq $sPartion)
        {
            return $aMount->{Mountpoint};
        }
    }
    return "";
}

# ------------------------------------------------------------------------------
sub getDiskSize($)
{
    my $sPartion = shift;
    my $Disk;
    foreach $Disk (@Disks)
    {
        if ($sPartion =~ /^$Disk->{Source}/)
        {
            return $Disk->{SizeInBytes};
        }
    }
    print "getDiskSize('$sPartion'): This should never occur, because for every disk the size should known!\n";
    exit(1);
    return 0;
}
sub getPossibleMountpoint($)
{
    my $sPartion = shift;
    my $aFstabEntry;
    foreach $aFstabEntry (@InFstab)
    {
        if ($aFstabEntry->{Source} eq $sPartion)
        {
            return $aFstabEntry->{Mountpoint};
        }
    }
    return "";
}
# ------------------------------------------------------------------------------
# open fdisk -l and put all found disk into @Disks array and all partions in @Partions array
sub identifyDiskAndPartions()
{
    # get all accessable disks
    local *FDISK;
    my $line;
    
    # identify all external disk
    open(FDISK, "LC_ALL=C fdisk -l |");
    my $whocares;
    my $sSource = "nerma";
    my $nSize;
    my $sSizeType;
    my $nSizeInBytes;
    
    while ($line=<FDISK>)
    {
        chomp($line);
        # we assume this format
        # Disk /dev/hda: 80.0 GB, 80026361856 bytes
        if ($line =~ /^Disk/)
        {
            # find a disk
            ($whocares,$sSource,$nSize,$sSizeType,$nSizeInBytes,$whocares) = split(' ', $line, 6);
            $sSource =~ s/:$//;   # remove leading ':'
            $sSizeType =~ s/,$//; # remove leading ','
            # my $Disk;
            my $Disk = {
                Source      => $sSource,
                SizeInBytes => $nSizeInBytes,
                SizeType    => $sSizeType,
                Size        => $nSize
                };
            
            push @Disks, $Disk;
        }
        elsif ($line =~ /^$sSource/)
        {
            if (! ($line =~ /Extended$/) )
            {
                my $sMountpoint;
                my $sRest;
                ($sMountpoint,$sRest) = split(' ', $line, 2);
                $sRest =~ s/^\*\s+//; # remove 'boot' asterix
                my $nBegin;
                my $nEnd;
                my $nSize;
                my $nType;
                my $sType;
                ($nBegin,$nEnd,$nSize,$nType,$sType) = split(' ', $sRest, 5);
                $nSize =~ s/\+$//; # remove leading '+'
                $nSize *= 1024;

                # find a partion
                my $aPartion = {
                    Source => $sMountpoint,
                    Size   => $nSize,
                    TypeNo => $nType,
                    Type   => $sType
                    };
                if ($verbose)
                {
                    print "partion: $sMountpoint";
                }
                push @Partions, $aPartion;
            }
        }
    }
    close(FDISK);
}
# ------------------------------------------------------------------------------
sub identifyAlreadyMounted()
{
    # get all accessable disks
    local *MOUNT;
    my $line;
    
    # identify all external disk
    open(MOUNT, "LC_ALL=C mount -v |");
    my $sSource = "nerma";
    my $sON;
    my $sMountpoint;
    my $sTYPE;
    my $sType;
    my $sParameter;
    
    while ($line=<MOUNT>)
    {
        chomp($line);
        # we assume this format
        # /dev/hda5 on / type ext3 (rw,noatime)
        # find a disk
        ($sSource,$sON,$sMountpoint,$sTYPE,$sType,$sParameter) = split(' ', $line);
        my $Mount = {
            Source      => $sSource,
            Mountpoint  => $sMountpoint,
            Type        => $sType,
            Parameter   => $sParameter
            };
        
        push @AlreadyMount, $Mount;
    }
    close(MOUNT);
}

# ------------------------------------------------------------------------------
sub identifyInFstab()
{
    # get all accessable disks
    local *FSTAB;
    my $line;
    
    # identify all external disk
    open(FSTAB, "/etc/fstab");
    my $sSource = "nerma";
    my $sMountpoint;
    my $sType;
    my $sParameter;
    my $a;
    my $b;

    while ($line=<FSTAB>)
    {
        chomp($line);
        # we assume this format
        # /dev/hda2		/boot		    ext2	noauto,noatime	1 2
        # find a disk
        if ($line =~ /^\#/ || $line eq "")
        {
            next;
        }

        ($sSource,$sMountpoint,$sType,$sParameter,$a,$b) = split(/\s+/, $line);
        my $Mount = {
            Source      => $sSource,
            Mountpoint  => $sMountpoint,
            Type        => $sType,
            Parameter   => $sParameter
            };
        
        push @InFstab, $Mount;
    }
    close(FSTAB);
}

# ----------------------------------------------------------------------------
sub print_usage(*)
{
    local *HANDLE = $_[0];
    my $tool_name = basename($0);

    print(HANDLE <<END_OF_USAGE);

Usage: $tool_name [OPTIONS] [Number]

Helper tool, to show all possible mountable and already mounted devices, which are not
CD or DVD devices. 
All mountable devices will show and start by a number, with a given number it is possible
to mount only one device.

  -m, --mount [num]    mount all not already mounted devices to /mnt/devicename
                       or if exist to it's mountpoint. If num is given, mount only
                       this device.
  -u, --unmount        unmount all herewith mounted devices

  -h, --help           Print this help, then exit
  -v, --version        Print version number, then exit

END_OF_USAGE
    ;
}
